/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.glite.data.transfer.fts;

import org.glite.data.transfer.*;

/**
 * The service interface of the File Transfer component. 
 * 
 * It exposes all the required functionality to submit, manage and query
 * transfer jobs, each containing one or many file transfers.
 *
 */
public interface FileTransfer extends ServiceBase {
	
    /**
     * Submits the placement job to the service. The job may be rejected
     * because the service has stopped accepting transfer jobs, because
     * the job description is malformed, or because the client is not
     * authorized for some of the files. The job's state is updated to
     * SUBMITTED upon the service.
     *
     * @param job The description of the files to transfer.
     * @return The request ID (the same as input parameter).
     *
     * @throws InvalidArgumentException        in the case of job being malformed.
     * @throws AuthorizationException          in the case of the user not being authorized to submit a job.
     * @throws InternalException               in the case of internal server errors (like the database being offline)
     * @throws ServiceBusyException            in the case of the service being temporarily unable to handle the request (queue full, being drained).
     *
     * <dt>
         * <br><b>Semantic description of the method:</b>
         * <br>The following steps are performed for each entry:
         * <ul>
         * <li> Check if the user is authorized to submit;
         * <br> if permission is not granted, throws {@link org.glite.data.transfer.AuthorizationException}.
         * <li> Checks if PlacementJob object is valid;
         * <br> if it is not valid, throws {@link org.glite.data.transfer.InvalidArgumentException}
         * <li> Check if the service is available;
         * <li> if the queue is full or the service is being drained, throw  {@link org.glite.data.transfer.ServiceBusyException}
         * <li> Queue the job.
         * </ul>
     */
    public String placementSubmit(PlacementJob job) throws InvalidArgumentException, AuthorizationException, ServiceBusyException, InternalException;

    /**
     * The same as placementSubmit except that it expects a delegation to have been made before.
     */
    public String placementSubmit2(PlacementJob job) throws InvalidArgumentException, AuthorizationException, ServiceBusyException, InternalException;

    /**
     * Submits the transfer job to the service. The job may be rejected
     * because the service has stopped accepting transfer jobs, because
     * the job description is malformed, or because the client is not
     * authorized for some of the files. The job's state is updated to
     * SUBMITTED upon the service.
     *
     * @param job The description of the files to transfer.
     * @return The request ID (the same as input parameter).
     *
     * @throws InvalidArgumentException        in the case of job being malformed.
     * @throws AuthorizationException          in the case of the user not being authorized to submit a job.
     * @throws InternalException               in the case of internal server errors (like the database being offline)
     * @throws ServiceBusyException            in the case of the service being temporarily unable to handle the request (queue full, being drained).
     *
     * <dt>
         * <br><b>Semantic description of the method:</b>
         * <br>The following steps are performed for each entry:
         * <ul>
         * <li> Check if the user is authorized to submit;
         * <br> if permission is not granted, throws {@link org.glite.data.transfer.AuthorizationException}.
         * <li> Checks if TransferJob object is valid;
         * <br> if it is not valid, throws {@link org.glite.data.transfer.InvalidArgumentException}
         * <li> Check if the service is available;
         * <li> if the queue is full or the service is being drained, throw  {@link org.glite.data.transfer.ServiceBusyException}
         * <li> Queue the job.
         * </ul>
     */
    public String transferSubmit(TransferJob job) throws InvalidArgumentException, AuthorizationException, ServiceBusyException, InternalException;

    /**
     * The same as transferSubmit except that it expects a delegation to have been made before. 
     */
    public String transferSubmit2(TransferJob job) throws InvalidArgumentException, AuthorizationException, ServiceBusyException, InternalException;

    /** 
     * The same as transferSubmit2 except that it takes optional checksum
     * values for each file and uses TransferJobElement2.
     */
    public String transferSubmit3(TransferJob2 job) throws InvalidArgumentException, AuthorizationException, ServiceBusyException, InternalException;

    /**
     * Submits the transfer job to the service. The job may be rejected
     * because the service has stopped accepting transfer jobs, because
     * the job description is malformed, or because the client is not
     * authorized for some of the files. The job's state is updated to
     * SUBMITTED upon the service.
     *
     * @param job The description of the files to transfer.
     * @return The request ID (the same as input parameter).
     *
     * @throws InvalidArgumentException        in the case of job being malformed.
     * @throws AuthorizationException          in the case of the user not being authorized to submit a job.
     * @throws InternalException               in the case of internal server errors (like the database being offline)
     * @throws ServiceBusyException            in the case of the service being temporarily unable to handle the request (queue full, being drained).
     * 
     * @deprecated Renamed {@link #transferSubmit(TransferJob)}. Placement (logical) jobs should go through the {@link #placementSubmit(PlacementJob)} method.
     *
     * <dt>
         * <br><b>Semantic description of the method:</b>
         * <br>The following steps are performed for each entry:
         * <ul>
         * <li> Check if the user is authorized to submit;
         * <br> if permission is not granted, throws {@link org.glite.data.transfer.AuthorizationException}.
         * <li> Checks if TransferJob object is valid;
         * <br> if it is not valid, throws {@link org.glite.data.transfer.InvalidArgumentException}
         * <li> Check if the service is available;
         * <li> if the queue is full or the service is being drained, throw  {@link org.glite.data.transfer.ServiceBusyException}
         * <li> Queue the job.
         * </ul>
     */
    public String submit(TransferJob job) throws InvalidArgumentException, AuthorizationException, ServiceBusyException, InternalException;
    
    /**
     * Return a list of jobIDs matching the given states on the given channel.
     *
     * @param inGivenStates states to match
     * @param channelName the channel name to match, if null, empty, or 'any', any channel will be matched.
     *
     * @return JobState listing the jobIDs and their associated states
     *
     * @throws org.glite.data.transfer.InvalidArgumentException        in the case of an unknow state request
     * @throws org.glite.data.transfer.AuthorizationException          in the case of the user not being authorized to request the state.
     * @throws org.glite.data.transfer.InternalException               in the case of internal server errors (like the database being offline)
     * @throws org.glite.data.transfer.ServiceBusyException            in the case of the service being temporarily unable to handle the request.
     *
     * @see org.glite.data.transfer.JobStatus
     */
    public JobStatus[] listRequests(String inGivenStates[], String channelName) throws InvalidArgumentException, AuthorizationException, ServiceBusyException, InternalException;

    /**
     * Return a list of jobIDs matching the given states on the given channel.
     *
     * @param inGivenStates states to match
     * @param channelName the channel name to match, if null, empty, or 'any', any channel will be matched.
     * @param forDN restrict to this DN
     * @param forVO restrict to this VO
     * 
     * @return JobState listing the jobIDs and their associated states
     *
     * @throws org.glite.data.transfer.InvalidArgumentException        in the case of an unknow state request
     * @throws org.glite.data.transfer.AuthorizationException          in the case of the user not being authorized to request the state.
     * @throws org.glite.data.transfer.InternalException               in the case of internal server errors (like the database being offline)
     * @throws org.glite.data.transfer.ServiceBusyException            in the case of the service being temporarily unable to handle the request.
     *
     * @see org.glite.data.transfer.JobStatus
     */
    public JobStatus[] listRequests2(String inGivenStates[], String channelName, String forDN, String forVO) throws InvalidArgumentException, AuthorizationException, ServiceBusyException, InternalException;

    /**
     * Return the statuses of all the files with the given requestID. It has a limit and offset
     * to allow chunked iteration through the file statuses for large jobs.
     *
     * @param requestID The requestID to obtain an Iterator over.
     * @param offset the offset from the first file to return
     * @param limit  the number of files to return
     * @return An array of FileTransferStatuses, one for each file
     * @throws org.glite.data.transfer.NotExistsException           in the case of unknown requestID.
     * @throws org.glite.data.transfer.ServiceBusyException         in the case of the service being temporarily unable to handle the request.
     * @throws org.glite.data.transfer.AuthorizationException       in the case of unauthorized use.
     * @throws org.glite.data.transfer.InternalException            in the case of the service not being available for some internal reason.
     * 
     * @see org.glite.data.transfer.FileTransferStatus
     */
    public FileTransferStatus[] getFileStatus(String requestID, int offset, int limit) throws NotExistsException, ServiceBusyException, InternalException, AuthorizationException;

    /**
     * Return the statuses of all the files with the given requestID. It has a limit and offset
     * to allow chunked iteration through the file statuses for large jobs.
     *
     * @param requestID The requestID to obtain an Iterator over.
     * @param offset the offset from the first file to return
     * @param limit  the number of files to return
     * @return An array of FileTransferStatus2es, one for each file
     * @throws org.glite.data.transfer.NotExistsException           in the case of unknown requestID.
     * @throws org.glite.data.transfer.ServiceBusyException         in the case of the service being temporarily unable to handle the request.
     * @throws org.glite.data.transfer.AuthorizationException       in the case of unauthorized use.
     * @throws org.glite.data.transfer.InternalException            in the case of the service not being available for some internal reason.
     * 
     * @see org.glite.data.transfer.FileTransferStatus2
     */
    public FileTransferStatus2[] getFileStatus2(String requestID, int offset, int limit) throws NotExistsException, ServiceBusyException, InternalException, AuthorizationException;

    /**
     * Get the summary state for the whole job.
     *
     * @param requestID The job request ID.
     * @return The state of the job.
     * @throws org.glite.data.transfer.NotExistsException       in the case of unknown requestID.
     * @throws org.glite.data.transfer.AuthorizationException   in the case of unauthorized use.
     * @throws org.glite.data.transfer.InternalException        in the case of the service not being available for some internal reason.
     * @throws org.glite.data.transfer.ServiceBusyException     in the case of the service being temporarily unable to handle the request.
     * 
     * @see org.glite.data.transfer.JobStatus
     * 
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check if the user is authorized to query the job;
     * <br> if permission is not granted, throws {@link org.glite.data.transfer.AuthorizationException}.
     * <li> Checks if TransferJob ID is there;
     * <br> if it is not, throws {@link org.glite.data.transfer.NotExistsException}
     * <li> Query the job state, return state.
     * </ul>
     */
    public JobStatus getTransferJobStatus(String requestID) throws NotExistsException, InternalException, AuthorizationException, ServiceBusyException;

    /**
     * Returns a summary status of the job. Including the job state and some
     * summary statistics.
     *
     * @param requestID The job request ID.
     * @return The summary of the job.
     * @throws org.glite.data.transfer.NotExistsException       in the case of unknown requestID.
     * @throws org.glite.data.transfer.AuthorizationException   in the case of unauthorized use.
     * @throws org.glite.data.transfer.InternalException        in the case of the service not being available for some internal reason.
     * @throws org.glite.data.transfer.ServiceBusyException     in the case of the service being temporarily unable to handle the request.
     * 
     * @see org.glite.data.transfer.TransferJobSummary
     * 
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check if the user is authorized to query the job;
     * <br> if permission is not granted, throws {@link org.glite.data.transfer.AuthorizationException}.
     * <li> Checks if TransferJob ID is there;
     * <br> if it is not, throws {@link org.glite.data.transfer.NotExistsException}
     * <li> Query the job, return summary.
     * </ul>
     */
    public TransferJobSummary getTransferJobSummary(String requestID) throws NotExistsException, InternalException, AuthorizationException, ServiceBusyException;

    /**
     * Returns the extended summary status of the job. Including the job state and some
     * summary statistics.
     *
     * @param requestID The job request ID.
     * @return The summary of the job.
     * @throws org.glite.data.transfer.NotExistsException       in the case of unknown requestID.
     * @throws org.glite.data.transfer.AuthorizationException   in the case of unauthorized use.
     * @throws org.glite.data.transfer.InternalException        in the case of the service not being available for some internal reason.
     * @throws org.glite.data.transfer.ServiceBusyException     in the case of the service being temporarily unable to handle the request.
     *
     * @see org.glite.data.transfer.TransferJobSummary2
     *
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check if the user is authorized to query the job;
     * <br> if permission is not granted, throws {@link org.glite.data.transfer.AuthorizationException}.
     * <li> Checks if TransferJob ID is there;
     * <br> if it is not, throws {@link org.glite.data.transfer.NotExistsException}
     * <li> Query the job, return summary.
     * </ul>
     */
    public TransferJobSummary2 getTransferJobSummary2(String requestID) throws NotExistsException, InternalException, AuthorizationException, ServiceBusyException;


    /**
     * Attempt to cancel the given jobs on the service.
     * <br><br>
     * The only semantics defined are 'best effort':<br>
     * a) no roll back. Any file already transferred will be kept
     * and not deleted; <br>b) any file currently transferring will complete and
     * will not be deleted. <br>c) Any files pending will be cancelled and will
     * not be transferred.
     *
     * Cancel has different semantics in case of the current job state:
     * <ul>
     * <li> SUBMITTED: job is immediately cancelled and moves to the FINISHED_DIRTY state.
     * <li> PENDING, ACTIVE: the job state is put to CANCELLING while all current transfers in the job are aborted.
            Once this is done, the job  state is CANCELLED. The individual file states may be
            DONE, TRANSFERFAILED and CANCELLED. For all files that have been successfully transferred (DONE), the registration
            is still performed. The final job state is FINISHED_DIRTY and the final individual file states are
            FINISHED, TRANSFERFAILED and CANCELLED. All FINISHED files have been registered correctly in the catalog.
            In rare cases it may happen that the cancle request comes just when the job is about to finish
            and the final state is FINISHED with all files in the DONE case.
     * </ul>
     *
     * @param requestIDs The job request IDs.
     *
     * @throws org.glite.data.transfer.NotExistsException       in the case of unknown requestID.
     * @throws org.glite.data.transfer.AuthorizationException   in the case of unauthorized use.
     * @throws org.glite.data.transfer.CannotCancelException    in the case of the job state being not cancellable.
     * @throws org.glite.data.transfer.InternalException        in the case of the service not being available for some internal reason.
     * @throws org.glite.data.transfer.ServiceBusyException     in the case of the service being temporarily unable to handle the request.
     * 
     * 
     * <dt>
     * <br><b>Semantic description of the method:</b>
     * <br>The following steps are performed for each entry:
     * <ul>
     * <li> Check if the user is authorized to query the job;
     * <br> if permission is not granted, throws {@link org.glite.data.transfer.AuthorizationException}.
     * <li> Checks if TransferJob ID is there;
     * <br> if it is not, throws {@link org.glite.data.transfer.NotExistsException}
     * <li> Query the job, return summary.
     * </ul>
     */
    public void cancel(String[] requestIDs) throws NotExistsException, InternalException, AuthorizationException, ServiceBusyException, CannotCancelException;
    
    /**
     * Update the priority of the job in the queue.
     *
     * This operation should be performed by ADMIN and VO MANAGER users.
     *
     * The priority value should be between 0 and 5.
     *
     * @param requestID The id of the job to set the priority
     * @param priority The new priority for the job (value between 0 and 5)
     * 
     * @throws org.glite.data.transfer.NotExistsException       in the case of unknown requestID.
     * @throws org.glite.data.transfer.AuthorizationException   in the case of unauthorized use.
     * @throws org.glite.data.transfer.InvalidArgumentException in the case of an invalid priority being given.
     * @throws org.glite.data.transfer.InternalException        in the case of the service not being available for some internal reason.
     * @throws org.glite.data.transfer.ServiceBusyException     in the case of the service being temporarily unable to handle the request.
     */
    public void setJobPriority(String requestID, int priority) throws AuthorizationException, NotExistsException, InvalidArgumentException, ServiceBusyException, InternalException;
    
    /**
     * Adds a manager to the given VO.
     * 
     * @param VOName the name of the vo on which to add the manager
     * @param principal the principal to be added as a manager
     * 
     * @throws AuthorizationException
     * @throws InvalidArgumentException
     * @throws NotExistsException
     * @throws ExistsException
     * @throws InternalException
     * 
     */
    public void addVOManager(String VOName, String principal) throws AuthorizationException, InvalidArgumentException, NotExistsException, ExistsException, InternalException;
    
    /**
     * Removes a manager from the given VO.
     * 
     * @param VOName the name of the vo from which to remove the manager
     * @param principal the principal to be removed
     * 
     * @throws AuthorizationException
     * @throws InvalidArgumentException
     * @throws NotExistsException
     * @throws InternalException
     * 
     */
    public void removeVOManager(String VOName, String principal) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
    
    /**
     * Lists managers of a given VO
     * 
     * @param VOName the name of the vo for which to list managers
     * 
     * @throws AuthorizationException
     * @throws InvalidArgumentException
     * @throws NotExistsException
     * @throws InternalException
     * @return an array of VO managers
     */
    public String[] listVOManagers(String VOName) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
        
    /**
     * Returns the roles of the client on this service.
     * 
     * @return A Roles object.
     * 
     * @throws AuthorizationException
     * @throws InternalException
     * 
     */
    public Roles getRoles() throws AuthorizationException, InternalException;

    /**
     * Returns the roles of the client on this service.
     *
     * @param otherDN the DN to check the roles for
     * @return A Roles object.
     *
     * @throws AuthorizationException
     * @throws InternalException
     *
     */
    public Roles getRolesOf(String otherDN) throws AuthorizationException, InternalException;

}
