/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.glite.data.transfer.channel;

import org.glite.data.transfer.*;

import java.util.Calendar;


/**
 * Interface for managing channels.
 */
public interface ChannelManagement extends ServiceBase {
    /**
     * Creates a new channel in the database.
     *
     * @param channel The initial channel description
     *
     * @throws AuthorizationException Not allowed to add a channel on the service
     * @throws InvalidArgumentException The information for the new channel is invalid
     * @throws InternalException Any other error (server side)
     *
     * @see org.glite.data.transfer.Channel
     */
    public void add(Channel channel) throws AuthorizationException, ExistsException, InvalidArgumentException, InternalException;

    /**
     * Delete a channel.
     *
     * @param name the channel to delete.
     *
     * @throws AuthorizationException Not allowed to remove the channel on the service
     * @throws NotExistsException There is no channel with such a name
     * @throws InternalException Any other error (server side)
     */
    public void drop(String name) throws AuthorizationException, NotExistsException, InternalException;

    /**
     * Get the description of the given channel.
     *
     * @param channelName The name of the channel to retrieve info
     *
     * @return The Channel object holding the channel info
     *
     * @throws AuthorizationException Not allowed to get information about the channel
     * @throws NotExistsException There is no channel with such a name
     * @throws InternalException Any other error (server side)
     *
     * @see org.glite.data.transfer.Channel
     */
    public Channel getChannel(String channelName) throws AuthorizationException, NotExistsException, InternalException;
    public Channel2 getChannel2(String channelName) throws AuthorizationException, NotExistsException, InternalException;
    public Channel3 getChannel3(String channelName) throws AuthorizationException, NotExistsException, InternalException;

    /**
     * List all existing channels.
     *
     * @throws AuthorizationException Not allowed to list channels on service
     * @throws InternalException Any other error (server side)
     *
     * @return An array of strings containing the channel names
     */
    public String[] listChannels() throws AuthorizationException, InternalException;

    /**
     * Manual intervention command on channel.
     * It changes the file_state for all files belonging to the given job from Hold
     * to the given file state. It will only work for files in the Hold state. The only
     * permissible requested states are 'Pending', 'Failed' or 'Canceling'.
     *
     * @param jobID The jobId of the job where the 'Hold' files state should be changed
     * @param state The new state for the files
     *
     * @throws AuthorizationException Not allowed to update file states on given job
     * @throws NotExistsException There is no job with such ID
     * @throws InternalException Any other error (server side)
     *
     */
    public void changeStateForHeldJob(String jobID, String state) throws AuthorizationException, NotExistsException, InternalException;

    /**
     * Manual intervention command on channel.
     * It changes the file_state for all files belonging to all jobs on the given channel
     * which are in the Hold state to the given file state.
     * It will only work for files in the Hold state. The only
     * permissible requested states are 'Pending', 'Failed' or 'Canceling'.
     *
     * @param channelName The channel name whose files state should be changed
     * @param state The new state for the files
     *
     * @throws AuthorizationException Not allowed to update file states for jobs on given channel
     * @throws NotExistsException There is no channel with such a name
     * @throws InternalException Any other error (server side)
     */
    public void changeStateForHeldJobsOnChannel(String channelName, String state) throws AuthorizationException, NotExistsException, InternalException;

    /**
     * Utility method to set the number of concurrent streams on a channel.
     *
     * @param channelName Name of channel to update info
     * @param numberOfStreams New value
     *
     * @throws AuthorizationException Not allowed to update info of given channel
     * @throws InvalidArgumentException The new value given is invalid
     * @throws NotExistsException There is no channel with such a name
     * @throws InternalException Any other error (server side)
     *
     * @see org.glite.data.transfer.Channel
     */
    public void setNumberOfStreams(String channelName, int numberOfStreams) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
    public void setNumberOfStreams2(String channelName, int numberOfStreams, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
    /**
     * Utility method to set the number of concurrent files on a channel
     *
     * @param channelName Name of channel to update info
     * @param numberOfFiles New value
     *
     * @throws AuthorizationException Not allowed to update info of given channel
     * @throws InvalidArgumentException The new value given is invalid
     * @throws NotExistsException There is no channel with such a name
     * @throws InternalException Any other error (server side)
     *
     * @see org.glite.data.transfer.Channel
     */
    public void setNumberOfFiles(String channelName, int numberOfFiles) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
    public void setNumberOfFiles2(String channelName, int numberOfFiles, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

    /**
     * Set the operational state of a channel.
     *
     * @param channelName Name of channel to update info
     * @param state New state
     *
     * @throws AuthorizationException Not allowed to update info of given channel
     * @throws InvalidArgumentException The new value given is invalid
     * @throws NotExistsException There is no channel with such a name
     * @throws InternalException Any other error (server side)
     *
     * @see org.glite.data.transfer.Channel
     */
    public void setState(String channelName, String state) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
    public void setState2(String channelName, String state, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

    /**
     * Set the capacity bandwidth (Mbits/s).
     *
     * @param channelName Name of channel to update info
     * @param utilisation New bandwith
     *
     * @throws AuthorizationException Not allowed to update info of given channel
     * @throws InvalidArgumentException The new value given is invalid
     * @throws NotExistsException There is no channel with such a name
     * @throws InternalException Any other error (server side)
     *
     * @see org.glite.data.transfer.Channel
     */
    public void setBandwidth(String channelName, int utilisation) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
    public void setBandwidth2(String channelName, int utilisation, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

    /**
     * Set the contact email for a channel.
     *
     * @param channelName Name of channel to update info
     * @param contact New contacominal throughtput targett
     *
     * @throws AuthorizationException Not allowed to update info of given channel
     * @throws InvalidArgumentException The new value given is invalid
     * @throws NotExistsException There is no channel with such a name
     * @throws InternalException Any other error (server side)
     *
     * @see org.glite.data.transfer.Channel
     */
    public void setContact(String channelName, String contact) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
    public void setContact2(String channelName, String contact, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

    /**
     * Set the nominal throughtput target for the channel.
     *
     * @param channelName Name of channel to update info
     * @param nominalThroughput New value
     *
     * @throws AuthorizationException Not allowed to update info of given channel
     * @throws InvalidArgumentException The new value given is invalid
     * @throws NotExistsException There is no channel with such a name
     * @throws InternalException Any other error (server side)
     *
     * @see org.glite.data.transfer.Channel
     */
    public void setNominalThroughput(String channelName, int nominalThroughput) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
    public void setNominalThroughput2(String channelName, int nominalThroughput, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

    /**
     * Set the TCP bufer size for the channel.
     *
     * @param channelName Name of channel to update info
     * @param bufferSizeString New value
     *
     * @throws AuthorizationException Not allowed to update info of given channel
     * @throws InvalidArgumentException The new value given is invalid
     * @throws NotExistsException There is no channel with such a name
     * @throws InternalException Any other error (server side)
     *
     * @see org.glite.data.transfer.Channel
     */
    public void setTCPBufferSize2(String channelName, String bufferSizeString, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

    /**
     * Adds a manager to the given channel.
     *
     * @param channelName channel name on which to add the manager
     * @param principal principal of the manager
     *
     * @throws AuthorizationException Not allowed to add manager on given channel
     * @throws InvalidArgumentException The given principal is invalid
     * @throws NotExistsException There is no channel with such a name
     * @throws ExistsException Given principal is already a channel manager
     * @throws InternalException Any other error (server side)
     */
    public void addManager(String channelName, String principal) throws AuthorizationException, InvalidArgumentException, NotExistsException, ExistsException, InternalException;

    /**
     * Removes a manager from the given channel.
     *
     * @param channelName channel name from which to remove the manager
     * @param principal principal to be removed
     *
     * @throws AuthorizationException Not allowed to remove manager on given channel
     * @throws InvalidArgumentException The given principal is invalid
     * @throws NotExistsException There is no channel with such a name, or principal is not a manager of channel
     * @throws InternalException Any other error (server side)
     */
    public void removeManager(String channelName, String principal) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

    /**
     * Lists all managers of a given channel.
     *
     * @param channelName channel name for which to list managers
     *
     * @throws AuthorizationException Not allowed to list managers of given channel
     * @throws NotExistsException There is no channel with such a name
     * @throws InternalException Any other error (server side)
     */
    public String[] listManagers(String channelName) throws AuthorizationException, NotExistsException, InternalException;

    /**
     * Sets the share of a VO on a given channel.
     *
     * A negative value removes the VO share from the channel share list.
     *
     * @param channelName channel name for which to set the vo share
     * @param VOName vo for which to set the share
     * @param share the share value (negative removes the vo from the channel share list)
     *
     * @throws AuthorizationException Not allowed to set vo shares for given channel
     * @throws InvalidArgumentException The value for share is invalid
     * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
     * @throws InternalException Any other error (server side)
     */
    public void setVOShare(String channelName, String VOName, int share) throws AuthorizationException, NotExistsException, InvalidArgumentException, InternalException;

    /**
      * Sets the transfer limit for a VO on a given channel.
      *
      * A negative value removes the VO limit from the channel limit list.
      *
      * @param channelName channel name for which to set the vo share
      * @param VOName vo for which to set the share
      * @param limit the maximum number of transfers allowed for a VO
      *
      * @throws AuthorizationException Not allowed to set vo shares for given channel
      * @throws InvalidArgumentException The value for share is invalid
      * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
      * @throws InternalException Any other error (server side)
      */
     public void setVOLimit(String channelName, String VOName, int limit) throws AuthorizationException, NotExistsException, InvalidArgumentException, InternalException;

    /**
     * Returns the channel audit information for a channel
     * @param startTime the start time for the search
     * @param endTime  the end time for the search
     * @param restrictType restricts what channel update events are searched for
     * @param channelName the channel to search on
     * @param lastNentries if set, will retrun only the top N entries
     * @return an array of ChannelAuditEntry
     *
     * @throws AuthorizationException Not allowed to set vo shares for given channel
     * @throws InvalidArgumentException The value for share is invalid
     * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
     * @throws InternalException Any other error (server side)
     */
    public ChannelAuditEntry[] channelAudit(Calendar startTime, Calendar endTime, String restrictType, int lastNentries, String channelName) throws AuthorizationException, NotExistsException, InvalidArgumentException, InternalException;
    
   /** 
    * Sets number of seconds to mark activity on URL Copy for a given channel.
    * @param channelName channel name for which to set the vo share
	* @param UrlCopyFirstTxmarkTo number of seconds to take as a first activity window 
	* 
	* @throws AuthorizationException Not allowed to set vo shares for given channel
	* @throws InvalidArgumentException The value for share is invalid
	* @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
	* @throws InternalException Any other error (server side)
    *  
    */    
    public void setUrlCopyFirstTxmarkTo(String channelName, int UrlCopyFirstTxmarkTo, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;	
    
    
    /** 
     * 
     * Sets a flag if to check the target directory existence.
     * 
    * @param channelName channel name for which to set the vo share
	* @param TargetDirCheck flag to check destination directory 
	* 
	* @throws AuthorizationException Not allowed to set vo shares for given channel
	* @throws InvalidArgumentException The value for share is invalid
	* @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
	* @throws InternalException Any other error (server side)
    */
	public void setTargetDirCheck(String channelName, int TargetDirCheck, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
	 	
    /* Parameters from Channel3: */

   /** 
    * Sets the type of a channel: DEDICATED 
    * (point-to-point, seLimit is not considered) or NONDEDICATED (start or cloud).

    *
    * @param channelName channel name
    * @param channelType type of a channel
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setChannelType(String channelName, String channelType, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the default block size of a transfer, e.g. '1MB' or '1024kb'. This parameter is passed to the globus FTP client.

    *
    * @param channelName channel name
    * @param blockSize default block size of a transfer
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setBlockSize(String channelName, String blockSize, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the timeouts are in seconds.
    *
    * @param channelName channel name
    * @param httpTimeout timeouts are in seconds.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setHttpTimeout(String channelName, int httpTimeout, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the possible values are: DEBUG, INFO (default), ERROR.
    *
    * @param channelName channel name
    * @param transferLogLevel possible values are: DEBUG, INFO (default), ERROR.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setTransferLogLevel(String channelName, String transferLogLevel, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the ratio of SRM activity to number of transfer slots.
    *
    * @param channelName channel name
    * @param preparingFilesRatio ratio of SRM activity to number of transfer slots.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setPreparingFilesRatio(String channelName, double preparingFilesRatio, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the url copy put timeout.
    *
    * @param channelName channel name
    * @param urlCopyPutTimeout url copy put timeout.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setUrlCopyPutTimeout(String channelName, int urlCopyPutTimeout, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the url copy put done timeout.
    *
    * @param channelName channel name
    * @param urlCopyPutDoneTimeout url copy put done timeout.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setUrlCopyPutDoneTimeout(String channelName, int urlCopyPutDoneTimeout, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the url copy get timeout.
    *
    * @param channelName channel name
    * @param urlCopyGetTimeout url copy get timeout.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setUrlCopyGetTimeout(String channelName, int urlCopyGetTimeout, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the url copy get done timeout.
    *
    * @param channelName channel name
    * @param urlCopyGetDoneTimeout url copy get done timeout.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setUrlCopyGetDoneTimeout(String channelName, int urlCopyGetDoneTimeout, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the url copy transfer timeout.
    *
    * @param channelName channel name
    * @param urlCopyTransferTimeout url copy transfer timeout.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setUrlCopyTransferTimeout(String channelName, int urlCopyTransferTimeout, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the url copy transfer markers timeout.
    *
    * @param channelName channel name
    * @param urlCopyTransferMarkersTimeout url copy transfer markers timeout.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setUrlCopyTransferMarkersTimeout(String channelName, int urlCopyTransferMarkersTimeout, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the make dCache happy timeout.
    *
    * @param channelName channel name
    * @param urlCopyNoProgressTimeout make dCache happy timeout.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setUrlCopyNoProgressTimeout(String channelName, int urlCopyNoProgressTimeout, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the transfer timeout per megabyte.
    *
    * @param channelName channel name
    * @param urlCopyTransferTimeoutPerMB transfer timeout per megabyte.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setUrlCopyTransferTimeoutPerMB(String channelName, double urlCopyTransferTimeoutPerMB, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the sRM copy direction: PUSH, PULL.
    *
    * @param channelName channel name
    * @param srmCopyDirection sRM copy direction: PUSH, PULL.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setSrmCopyDirection(String channelName, String srmCopyDirection, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the this is a global timeout.
    *
    * @param channelName channel name
    * @param srmCopyTimeout this is a global timeout.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setSrmCopyTimeout(String channelName, int srmCopyTimeout, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

   /** 
    * Sets the timeout between status updates.
    *
    * @param channelName channel name
    * @param srmCopyRefreshTimeout timeout between status updates.
    * 
    * @throws AuthorizationException Not allowed to set vo shares for given channel
    * @throws InvalidArgumentException The value for share is invalid
    * @throws NotExistsException There is no channel with such a name, or there is no VO with such a name
    * @throws InternalException Any other error (server side)
    *  
    */    
    public void setSrmCopyRefreshTimeout(String channelName, int srmCopyRefreshTimeout, String message) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
    
    /**
     * Get a list of the names of the defined site groups.
     * @throws AuthorizationException Not allowed to list groups on service.
     * @throws InternalException Any other error (server side).
     * @return An array of strings containing the site groups names.
     */
    public String[] getSiteGroupNames() throws AuthorizationException, InternalException;

    /**
     * Get the list of the members of the site group with given name
     * @param groupName the name of the group.
     * @return Array of strings containing the names of the sites member of the group.
     * @throws AuthorizationException Not allowed to list group members.
     * @throws InvalidArgumentException The groupName parameter is invalid.
     * @throws NotExistsException There is no group with such a name.
     * @throws InternalException Any other error (server side).
     */
    public String[] getSiteGroupMembers(String groupName) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

    /**
     * Add the site with given name to members of the given group. If the group does not exist it is created.
     * @param groupName the name of the group.
     * @param siteName the name of the site.
     * @throws AuthorizationException Not allowed to add group members.
     * @throws InvalidArgumentException One of the provided parameters is invalid.
     * @throws NotExistsException There is no group with such a name.
     * @throws ExistsException Given site is already a member of the group.
     * @throws InternalException Any other error (server side).
     */
    public void addGroupMember(String groupName,String siteName) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;

    /**
     * Remove the site with given name from members of the given group. If after the operation the group is empty, the group is deleted.
     * @param groupName the name of the group.
     * @param siteName the name of the site.
     * @throws AuthorizationException Not allowed to remove group members.
     * @throws InvalidArgumentException One of the provided parameters is invalid.
     * @throws NotExistsException There is no group with such a name or site is not member of given group.
     * @throws InternalException Any other error (server side).
     */
    public void removeGroupMember(String groupName,String siteName) throws AuthorizationException, InvalidArgumentException, NotExistsException, InternalException;
}
